# Copyright 2022 The Wordcab Team. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Wordcab API Login."""

import logging
import os
from getpass import getpass
from pathlib import Path
from typing import Optional

import requests  # type: ignore

from .config import WORDCAB_TOKEN_FOLDER


logger = logging.getLogger(__name__)

path_to_token = Path(WORDCAB_TOKEN_FOLDER).expanduser()


def login(account: str, token: Optional[str] = None) -> None:
    """Login to Wordcab API."""
    if token is not None:
        _login(account, token)
    else:
        raise ValueError(
            "Please provide a token or use `wordcab login` command to login from CLI."
        )


def cli_login() -> None:
    """
    Login to Wordcab API from CLI.

    This function is called from the CLI, prompting the user for their
    credentials and then storing them as git credentials.
    """
    logger.warning(
        r"""
         _  _   __   ____  ____   ___   __   ____
        / )( \ /  \ (  _ \(    \ / __) / _\ (  _ \
        \ /\ /(  O ) )   / ) D (( (__ /    \ ) _ (
        (_/\_) \__/ (__\_)(____/ \___)\_/\_/(____/

        To login, please use your API token generated from https://wordcab.com/account/api-key/
        """
    )
    account = input("    Email: ")
    token = getpass("    API Token: ")
    _login(account, token)


def cli_logout() -> None:
    """
    Logout from Wordcab API from CLI.

    This function is called from the CLI, prompting the user for their email
    address and then removing their stored git credentials.
    """
    logger.warning(
        r"""
     _  _   __   ____  ____   ___   __   ____
    / )( \ /  \ (  _ \(    \ / __) / _\ (  _ \
    \ /\ /(  O ) )   / ) D (( (__ /    \ ) _ (
    (_/\_) \__/ (__\_)(____/ \___)\_/\_/(____/

    To logout, please enter your email address.
        """
    )
    account = input("    Email: ")
    _logout(account)


def get_token() -> Optional[str]:
    """Read API token from git credential store."""
    token = os.environ.get("WORDCAB_API_KEY")
    if token is None:
        try:
            return path_to_token.read_text().split(":")[1]
        except FileNotFoundError:
            pass
    return token


def _check_valid_credentials(account: str, token: str) -> bool:
    """Check if the credentials are valid."""
    headers = {"Authorization": f"Bearer {token}"}
    r = requests.get("https://wordcab.com/api/v1/me", headers=headers)
    if r.status_code == 200:
        if r.json()["account_email"] == account:
            return True
    return False


def _login(account: str, token: str) -> None:
    """Login to Wordcab API."""
    if _check_valid_credentials(account, token):
        path_to_token.parent.mkdir(exist_ok=True)
        with path_to_token.open("w+") as f:
            f.write(f"{account}:{token}")
        logger.warning(
            f"Successfully logged in as {account} . Credentials are stored in {path_to_token}"
        )
    else:
        logger.warning(
            f"Invalid credentials for {account} . Please check your API token."
        )


def _logout(account: str) -> None:
    """Logout from Wordcab API."""
    path_to_token.unlink()
    logger.warning(
        f"Successfully logged out as {account} . Credentials are removed from {path_to_token}"
    )



# Copyright 2022 The Wordcab Team. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Wordcab API Job object."""

import logging
from dataclasses import dataclass, field
from typing import Dict, List, Optional, Union

from ..config import (
    EXTRACT_AVAILABLE_STATUS,
    SOURCE_OBJECT_MAPPING,
    SUMMARIZE_AVAILABLE_STATUS,
)


logger = logging.getLogger(__name__)


@dataclass
class JobSettings:
    """Wordcab API Job Settings object."""

    ephemeral_data: Optional[bool] = field(default=False)
    pipeline: str = field(default="default")
    only_api: Optional[bool] = field(default=True)
    split_long_utterances: Optional[bool] = field(default=False)

    def __post_init__(self) -> None:
        """Post init."""
        if self.pipeline == "default":
            raise ValueError("Pipeline must be set to a valid pipeline name")


@dataclass
class BaseJob:
    """Wordcab API BaseJob object."""

    display_name: str
    job_name: str
    source: str
    job_status: Optional[str] = field(default="Pending")
    metadata: Optional[Dict[str, str]] = field(default=None)
    settings: Optional[JobSettings] = field(default=None)
    tags: Optional[List[str]] = field(default=None)
    time_started: Optional[str] = field(default=None)
    time_completed: Optional[str] = field(default=None)
    transcript_id: Optional[str] = field(default=None)

    def __post_init__(self) -> None:
        """Post-init method."""
        logger.info(f"Job {self.job_name} created.")
        if self.source not in SOURCE_OBJECT_MAPPING.keys():
            raise ValueError(
                f"""
                Source {self.source} is not a valid source. Valid sources are {SOURCE_OBJECT_MAPPING.keys()}.
            """
            )

    def job_update(self, parameters: Dict[str, str]) -> None:
        """Update the job attributes."""
        for key, value in parameters.items():
            if key in self.__dict__:
                if getattr(self, key) != value:
                    setattr(self, key, value)
                    logger.info(f"Job {self.job_name} updated: {key} = {value}")
                else:
                    logger.info(f"Job {self.job_name} not updated: {key} = {value}")
            else:
                logger.warning(
                    f"Cannot update {key} in {self.job_name}, not a valid attribute."
                )


@dataclass
class ExtractJob(BaseJob):
    """Wordcab API ExtractJob object."""

    def __post_init__(self) -> None:
        """Post-init."""
        super().__post_init__()
        self._job_type = "ExtractJob"
        self.available_status = EXTRACT_AVAILABLE_STATUS


@dataclass
class SummarizeJob(BaseJob):
    """Wordcab API SummarizeJob object."""

    summary_details: Optional[Dict[str, str]] = field(default=None)

    def __post_init__(self) -> None:
        """Post-init."""
        super().__post_init__()
        self._job_type = "SummarizeJob"
        self.available_status = SUMMARIZE_AVAILABLE_STATUS


@dataclass
class ListJobs:
    """Wordcab API ListJobs object."""

    page_count: int
    next_page: str
    results: List[Union[ExtractJob, SummarizeJob]]



# Copyright 2022 The Wordcab Team. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Wordcab API mapping functions."""

from typing import Dict, List, Optional, Union, no_type_check

from .client import Client
from .core_objects import (
    BaseSource,
    BaseSummary,
    BaseTranscript,
    ExtractJob,
    InMemorySource,
    ListJobs,
    ListSummaries,
    ListTranscripts,
    Stats,
    SummarizeJob,
)


@no_type_check
def request(
    method: str,
    api_key: Optional[str] = None,
    **kwargs: Union[bool, int, str, Dict[str, str], List[int], List[str]]
) -> Union[
    BaseSource,
    BaseSummary,
    BaseTranscript,
    ExtractJob,
    ListJobs,
    ListSummaries,
    ListTranscripts,
    Stats,
    SummarizeJob,
    Union[ExtractJob, SummarizeJob],
]:
    """Make a request to the Wordcab API."""
    with Client(api_key=api_key) as client:
        return client.request(method=method, **kwargs)


@no_type_check
def get_stats(
    min_created: Optional[str] = None,
    max_created: Optional[str] = None,
    tags: Optional[List[str]] = None,
    api_key: Optional[str] = None,
) -> Stats:
    """
    Retrieve account stats such as spend and request volume, by timestamp or tag.

    Parameters
    ----------
    min_created : str, optional
        The minimum limit of the specified time range. The default is None. If
        None, the minimum limit will be automatically set to a month prior.
    max_created : str
        The maximum limit of the specified time range. The default is None. If
        None, the maximum limit will be automatically set to the current time.
    tags : list of str, optional
        A list of tags to filter by. The default is None. If None, no tags will
        be used to filter the stats.
    api_key : str, optional
        The API key to use. The default is None. If None, the API key will be
        automatically retrieved from the environment variable WORDCAB_API_KEY.

    Returns
    -------
    Stats
        The stats object containing the stats data.
    """
    return request(
        method="get_stats",
        min_created=min_created,
        max_created=max_created,
        tags=tags,
        api_key=api_key,
    )


@no_type_check
def start_extract(
    source_object: Union[BaseSource, InMemorySource],
    display_name: str,
    ephemeral_data: Optional[bool] = False,
    only_api: Optional[bool] = True,
    pipelines: Union[str, List[str]] = [  # noqa: B006
        "questions_answers",
        "topic_segments",
        "emotions",
        "speaker_talk_ratios",
    ],
    split_long_utterances: bool = False,
    tags: Optional[Union[str, List[str]]] = None,
    api_key: Optional[str] = None,
) -> ExtractJob:
    """
    Start an extraction job.

    Parameters
    ----------
    source_object : BaseSource or InMemorySource
        The source object to use for the extraction job.
    display_name : str
        The display name of the extraction job. This is useful for retrieving the job later.
    ephemeral_data : bool, optional
        Whether to delete the data after the job is complete. The default is False. If False, the data will be
        kept on WordCab's servers. You can delete the data at any time, check the documentation here:
        https://docs.wordcab.com/docs/enabling-ephemeral-data
    only_api : bool, optional
        Whether to only use the API for the extraction job. The default is True.
    pipelines : list of str, optional
        A list of pipelines to use for the extraction job. The default is ["questions_answers", "topic_segments",
        "emotions", "speaker_talk_ratios"]. You can use one or more of the available pipelines.
    split_long_utterances : bool
        Whether to split long utterances into multiple shorter utterances. The default is False.
    tags : str or list of str, optional
        The tags to add to the job. The default is None. If None, no tags will be added.
    api_key : str, optional
        The API key to use. The default is None. If None, the API key will be
        automatically retrieved from the environment variable WORDCAB_API_KEY.

    Returns
    -------
    ExtractJob
        The extract job object.
    """
    return request(
        method="start_extract",
        source_object=source_object,
        display_name=display_name,
        ephemeral_data=ephemeral_data,
        only_api=only_api,
        pipelines=pipelines,
        split_long_utterances=split_long_utterances,
        tags=tags,
        api_key=api_key,
    )


@no_type_check
def start_summary(
    source_object: Union[BaseSource, InMemorySource],
    display_name: str,
    summary_type: str,
    ephemeral_data: bool = False,
    only_api: bool = True,
    pipelines: Union[str, List[str]] = ["transcribe", "summarize"],  # noqa: B006
    source_lang: Optional[str] = None,
    split_long_utterances: bool = False,
    summary_length: Union[int, List[int]] = 3,
    tags: Optional[Union[str, List[str]]] = None,
    api_key: Optional[str] = None,
) -> SummarizeJob:
    """
    Start a summary job.

    Parameters
    ----------
    source_object : BaseSource or InMemorySource
        The source object to summarize.
    display_name : str
        The display name of the summary. This is useful for retrieving the job later.
    summary_type : str
        The type of summary to create. You can choose from "conversational", "narrative", "reason_conclusion" or
        "no_speaker". More information can be found here: https://docs.wordcab.com/docs/summary-types
    ephemeral_data : bool
        Whether to delete the data after the summary is created. The default is False. If False, the data will be
        kept on Wordcab's servers. You can delete the data at any time, check the documentation here:
        https://docs.wordcab.com/docs/enabling-ephemeral-data
    only_api : bool
        Whether to only use the API to create the summary. The default is True.
    pipelines : str or list of str
        The pipelines to use. The default is ["transcribe", "summarize"].
    source_lang : str, optional
        The language of the source. If None, the language will be `en` (English) by default.
    split_long_utterances : bool
        Whether to split long utterances into multiple shorter utterances. The default is False.
    summary_length : int or list of int, optional
        The length of the summary. The default is 3. The length should be between 1 and 5. If a list of ints is
        provided, the summary will be created for each length.
    tags : str or list of str, optional
        The tags to add to the job. The default is None. If None, no tags will be added.
    api_key : str, optional
        The API key to use. The default is None. If None, the API key will be
        automatically retrieved from the environment variable WORDCAB_API_KEY.

    Returns
    -------
    SummarizeJob
        The summarize job object.
    """
    return request(
        method="start_summary",
        source_object=source_object,
        display_name=display_name,
        summary_type=summary_type,
        ephemeral_data=ephemeral_data,
        only_api=only_api,
        pipelines=pipelines,
        source_lang=source_lang,
        split_long_utterances=split_long_utterances,
        summary_length=summary_length,
        tags=tags,
        api_key=api_key,
    )


@no_type_check
def list_jobs(
    page_size: int = 100, order_by: str = "-time_started", api_key: Optional[str] = None
) -> ListJobs:
    """
    Retrieve a list of jobs.

    Parameters
    ----------
    page_size : int
        The number of jobs to retrieve per page. The default is 100.
    order_by : str
        The order to retrieve the jobs in. The default is "-time_started".
    api_key : str, optional
        The API key to use. The default is None. If None, the API key will be
        automatically retrieved from the environment variable WORDCAB_API_KEY.

    Returns
    -------
    ListJobs
        The list jobs object containing the list of jobs. The jobs can be
        SummarizeJob or ExtractJob objects.
    """
    return request(
        method="list_jobs", page_size=page_size, order_by=order_by, api_key=api_key
    )


@no_type_check
def retrieve_job(
    job_name: str, api_key: Optional[str] = None
) -> Union[ExtractJob, SummarizeJob]:
    """
    Retrieve a job by name.

    Parameters
    ----------
    job_name : str
        The name of the job to retrieve.
    api_key : str, optional
        The API key to use. The default is None. If None, the API key will be
        automatically retrieved from the environment variable WORDCAB_API_KEY.

    Returns
    -------
    ExtractJob or SummarizeJob
        The job object. The job can be an ExtractJob or SummarizeJob object.
    """
    return request(method="retrieve_job", job_name=job_name, api_key=api_key)


@no_type_check
def delete_job(job_name: str, api_key: Optional[str] = None) -> Dict[str, str]:
    """
    Delete a job by name and all associated data (including the transcript).

    Note that this will delete the transcript from WordCab's servers. If you want to keep the transcript,
    you should download it before deleting the job.

    Parameters
    ----------
    job_name: str
        The name of the job to delete.
    api_key : str, optional
        The API key to use. The default is None. If None, the API key will be
        automatically retrieved from the environment variable WORDCAB_API_KEY.

    Returns
    -------
    Dict[str, str]
        A dictionary containing the name of the deleted job.
    """
    return request(method="delete_job", job_name=job_name, api_key=api_key)


@no_type_check
def list_transcripts(
    page_size: int = 100, api_key: Optional[str] = None
) -> ListTranscripts:
    """
    Retrieve a list of transcripts.

    Parameters
    ----------
    page_size : int
        The number of transcripts to retrieve per page. The default is 100.
    api_key : str, optional
        The API key to use. The default is None. If None, the API key will be
        automatically retrieved from the environment variable WORDCAB_API_KEY.

    Returns
    -------
    ListTranscripts
        The list transcripts object containing the list of transcripts.
    """
    return request(method="list_transcripts", page_size=page_size, api_key=api_key)


@no_type_check
def retrieve_transcript(
    transcript_id: str, api_key: Optional[str] = None
) -> BaseTranscript:
    """
    Retrieve a transcript by id.

    Parameters
    ----------
    transcript_id : str
        The id of the transcript to retrieve.
    api_key : str, optional
        The API key to use. The default is None. If None, the API key will be
        automatically retrieved from the environment variable WORDCAB_API_KEY.

    Returns
    -------
    BaseTranscript
        The transcript object.
    """
    return request(
        method="retrieve_transcript", transcript_id=transcript_id, api_key=api_key
    )


@no_type_check
def change_speaker_labels(
    transcript_id: str, speaker_map: Dict[str, str], api_key: Optional[str] = None
) -> BaseTranscript:
    """
    Change speaker labels in a transcript.

    Parameters
    ----------
    transcript_id : str
        The id of the transcript to change the speaker labels of.
    speaker_map : Dict[str, str]
        A dictionary mapping the old speaker labels to the new speaker labels.
    api_key : str, optional
        The API key to use. The default is None. If None, the API key will be
        automatically retrieved from the environment variable WORDCAB_API_KEY.

    Returns
    -------
    BaseTranscript
        The transcript object with the changed speaker labels.
    """
    return request(
        method="change_speaker_labels",
        transcript_id=transcript_id,
        speaker_map=speaker_map,
        api_key=api_key,
    )


@no_type_check
def list_summaries(
    page_size: int = 100, api_key: Optional[str] = None
) -> ListSummaries:
    """
    Retrieve a list of summaries.

    Parameters
    ----------
    page_size : int
        The number of summaries to retrieve per page. The default is 100.
    api_key : str, optional
        The API key to use. The default is None. If None, the API key will be
        automatically retrieved from the environment variable WORDCAB_API_KEY.

    Returns
    -------
    ListSummaries
        The list summaries object containing the list of summaries.
    """
    return request(method="list_summaries", page_size=page_size, api_key=api_key)


@no_type_check
def retrieve_summary(summary_id: str, api_key: Optional[str] = None) -> BaseSummary:
    """
    Retrieve a summary by id.

    Parameters
    ----------
    summary_id : str
        The id of the summary to retrieve.
    api_key : str, optional
        The API key to use. The default is None. If None, the API key will be
        automatically retrieved from the environment variable WORDCAB_API_KEY.

    Returns
    -------
    BaseSummary
        The summary object.
    """
    return request(method="retrieve_summary", summary_id=summary_id, api_key=api_key)



# Copyright 2022 The Wordcab Team. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Wordcab API Config variables."""

AVAILABLE_AUDIO_FORMATS = [".flac", ".m4a", ".mp3", ".mpga", ".ogg", ".wav"]
AVAILABLE_GENERIC_FORMATS = [".json", ".txt"]
AVAILABLE_PLAN = ["free", "metered", "paid"]
EXTRACT_AVAILABLE_STATUS = [
    "Deleted",
    "Error",
    "Extracting",
    "ExtractionComplete",
    "ItemQueued",
    "Pending",
    "PreparingExtraction",
]
EXTRACT_PIPELINES = [
    "questions_answers",
    "topic_segments",
    "emotions",
    "speaker_talk_ratios",
]
LIST_JOBS_ORDER_BY = [
    "time_started",
    "time_completed",
    "-time_started",
    "-time_completed",
]
SOURCE_LANG = ["de", "en", "es", "fr", "it"]
SOURCE_OBJECT_MAPPING = {
    "generic": "GenericSource",
    "audio": "AudioSource",
    "wordcab_transcript": "WordcabTranscriptSource",
    "signed_url": "SignedUrlSource",
    "assembly_ai": "AssemblyAISource",
    "deepgram": "DeepgramSource",
    "rev_ai": "RevSource",
    "vtt": "VTTSource",
}
SUMMARIZE_AVAILABLE_STATUS = [
    "Deleted",
    "Error",
    "ItemQueued",
    "Pending",
    "PreparingSummary",
    "PreparingTranscript",
    "Summarizing",
    "SummaryComplete",
    "Transcribing",
    "TranscriptComplete",
]
SUMMARY_LENGTHS_RANGE = [1, 5]
SUMMARY_PIPELINES = ["transcribe", "summarize"]
SUMMARY_TYPES = ["conversational", "narrative", "no_speaker", "reason_conclusion"]
WORDCAB_TOKEN_FOLDER = "~/.wordcab/token"  # noqa: S105



# Copyright 2022 The Wordcab Team. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Wordcab API Transcript object."""

import logging
from dataclasses import dataclass, field
from typing import Dict, List, Optional


logger = logging.getLogger(__name__)


@dataclass
class TranscriptUtterance:
    """Transcript utterance object."""

    text: str
    speaker: str
    end: str
    start: str
    timestamp_end: int
    timestamp_start: int

    def __post_init__(self) -> None:
        """Post-init method."""
        if not isinstance(self.text, str):
            raise TypeError(
                f"TranscriptUtterance.text must be a string, not {type(self.text)}"
            )

        if not isinstance(self.speaker, str):
            raise TypeError(
                f"TranscriptUtterance.speaker must be a string, not {type(self.speaker)}"
            )

        if self.timestamp_end < self.timestamp_start:
            raise ValueError(
                f"""
                TranscriptUtterance.end_index must be greater than or equal to TranscriptUtterance.start_index,
                not {self.timestamp_end} < {self.timestamp_start}"""
            )


@dataclass
class BaseTranscript:
    """Transcript object."""

    transcript_id: str
    job_id_set: List[str] = field(default_factory=list)
    summary_id_set: List[str] = field(default_factory=list)
    transcript: List[TranscriptUtterance] = field(default_factory=list)
    speaker_map: Dict[str, str] = field(default_factory=dict)
    question_answers: Optional[List[Dict[str, str]]] = field(default=None)

    def __post_init__(self) -> None:
        """Post-init method."""
        if self.speaker_map:
            for key, val in self.speaker_map.items():
                if not isinstance(key, str):
                    raise TypeError(
                        f"BaseTranscript.speaker_map keys must be strings, not {type(key)}"
                    )
                if not isinstance(val, str):
                    raise TypeError(
                        f"BaseTranscript.speaker_map values must be strings, not {type(val)}"
                    )

    def update_speaker_map(self, speaker_map: Dict[str, str]) -> None:
        """Update the speaker map for the transcript."""
        self.speaker_map = speaker_map


@dataclass
class ListTranscripts:
    """List transcripts object."""

    page_count: int
    next_page: str
    results: List[BaseTranscript]



# Copyright 2022 The Wordcab Team. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Wordcab API Summary object."""

import logging
from dataclasses import dataclass, field
from typing import Any, Dict, List, Optional, Union

from ..config import SUMMARY_TYPES


logger = logging.getLogger(__name__)


@dataclass
class StructuredSummary:
    """Structured summary object."""

    summary: str
    summary_html: Optional[str] = field(default=None)
    end: Optional[str] = field(default=None)
    end_index: Optional[int] = field(default=None)
    start: Optional[str] = field(default=None)
    start_index: Optional[int] = field(default=None)
    timestamp_end: Optional[int] = field(default=None)
    timestamp_start: Optional[int] = field(default=None)
    transcript_segment: Optional[List[Dict[str, Union[str, int]]]] = field(default=None)

    def __repr__(self) -> str:
        """Return a string representation of the object without the None values."""
        return f"{self.__class__.__name__}({', '.join(f'{k}={v!r}' for k, v in self.__dict__.items() if v is not None)})"


@dataclass
class ReasonSummary:
    """Reason summary object."""

    summary: str
    stop_index: int


@dataclass
class ConclusionSummary:
    """Conclusion summary object."""

    summary: str
    stop_index: int


@dataclass
class BaseSummary:
    """Summary object."""

    job_status: str
    summary_id: str
    display_name: Optional[str] = field(default=None)
    job_name: Optional[str] = field(default=None)
    process_time: Optional[str] = field(default=None)
    speaker_map: Optional[Dict[str, str]] = field(default=None)
    source: Optional[str] = field(default=None)
    summary_type: Optional[str] = field(default=None)
    summary: Optional[Dict[str, Any]] = field(default=None)
    transcript_id: Optional[str] = field(default=None)
    time_started: Optional[str] = field(default=None)
    time_completed: Optional[str] = field(default=None)

    def __post_init__(self) -> None:
        """Post init."""
        if self.summary_type:
            if self.summary_type not in SUMMARY_TYPES:
                raise ValueError(
                    f"Summary type must be one of {SUMMARY_TYPES}, not {self.summary_type}"
                )

        if self.time_started and self.time_completed:
            if self.time_started == self.time_completed:
                raise ValueError("time_started and time_completed must be different")


@dataclass
class ListSummaries:
    """List summaries object."""

    page_count: int
    next_page: str
    results: List[BaseSummary]



# Copyright 2022 The Wordcab Team. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Command-line interface."""
import click

from .login import cli_login, cli_logout


@click.group()
@click.version_option()
def main() -> None:
    """Wordcab Python SDK."""
    pass


@click.command()
def login() -> None:
    """Prompt the user for API credentials and store them as git credentials."""
    cli_login()


@click.command()
def logout() -> None:
    """Remove stored git credentials."""
    cli_logout()


main.add_command(login)
main.add_command(logout)


if __name__ == "__main__":
    main(prog_name="wordcab")  # pragma: no cover



# Copyright 2022 The Wordcab Team. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Wordcab API Source object."""

import json
import logging
from dataclasses import dataclass, field
from pathlib import Path
from typing import Dict, List, Optional, Union, no_type_check

import validators  # type: ignore

from ..config import AVAILABLE_AUDIO_FORMATS, AVAILABLE_GENERIC_FORMATS


logger = logging.getLogger(__name__)


@dataclass
class BaseSource:
    """Base class for all source objects except for InMemorySource. It is not meant to be used directly.

    Parameters
    ----------
    filepath : Optional[Union[str, Path]], optional
        Path to the local file, by default None.
    url : Optional[str], optional
        URL to the remote file, by default None.

    Raises
    ------
    ValueError
        If neither `filepath` nor `url` are provided.
    ValueError
        If both `filepath` and `url` are provided.
    TypeError
        If `filepath` is not a string or a Path object.
    FileNotFoundError
        If `filepath` does not exist or is not accessible.

    Attributes
    ----------
    source : str
        The source type.
    source_type : str
        The source type.
    _stem : str
        The stem of the file.
    _suffix : str
        The suffix of the file.

    Returns
    -------
    BaseSource
        The source object.
    """

    filepath: Optional[Union[str, Path]] = field(default=None, repr=False)
    url: Optional[str] = field(default=None, repr=False)
    source: str = field(init=False)
    source_type: str = field(init=False)
    _stem: str = field(init=False, repr=False)
    _suffix: str = field(init=False, repr=False)

    def __post_init__(self) -> None:
        """Post-init method."""
        self.source = self.__class__.__name__
        if not self.filepath and not self.url:
            raise ValueError(
                "Please provide either a local or a remote source, respectively `filepath` or `url`."
            )
        if self.filepath and self.url:
            raise ValueError(
                "Please provide either a local or a remote source, not both `filepath` and `url`."
            )

        if self.filepath:
            if not isinstance(self.filepath, Path) and not isinstance(
                self.filepath, str
            ):
                raise TypeError(
                    f"The path must be a string or a Path object, not {type(self.filepath)}"
                )

            if isinstance(self.filepath, str):
                self.filepath = Path(self.filepath)

            if not self.filepath.exists():
                raise FileNotFoundError(
                    f"File {self.filepath} does not exist or is not accessible."
                )

            self._stem = self.filepath.stem
            self._suffix = self.filepath.suffix
            self.source_type = "local"

        if self.url:
            if not validators.url(self.url):
                raise ValueError(
                    f"Please provide a valid URL. {self.url} is not valid."
                )
            self.source_type = "remote"

    @no_type_check
    def _load_file_from_path(self) -> bytes:
        """Load file from local path."""
        with open(self.filepath, "rb") as f:
            return f.read()

    def _load_file_from_url(self) -> bytes:
        """Load file from URL."""
        raise NotImplementedError("Loading files from URLs is not implemented yet.")

    def prepare_payload(self) -> Union[str, Dict[str, bytes]]:
        """Prepare payload."""
        raise NotImplementedError("Payload preparation is not implemented yet.")

    def prepare_headers(self) -> Dict[str, str]:
        """Prepare headers."""
        raise NotImplementedError("Headers preparation is not implemented yet.")


@dataclass
class InMemorySource:
    """In-memory source object.

    The in-memory source object is a special case of the generic source object.
    It is used to pass a pre-processed transcript to the API.

    Parameters
    ----------
    obj : Union[Dict[str, List[str]], List[str]]
        The in-memory object. It can be a list of strings or a dict with a `transcript` key
        and a list of strings as value.

    Raises
    ------
    ValueError
        If the in-memory object does not have a `transcript` key.
    TypeError
        If the in-memory object does not have a list as value for the `transcript` key.
    TypeError
        If the in-memory object is not a list or a dict.

    Examples
    --------
    >>> from wordcab.core_objects import InMemorySource

    >>> transcript = {"transcript": ["SPEAKER A: Hello.", "SPEAKER B: Hi."]}
    >>> in_memory_source = InMemorySource(obj=transcript)
    >>> in_memory_source
    InMemorySource(...)
    >>> in_memory_source.obj

    Returns
    -------
    InMemorySource
        The in-memory source object.
    """

    obj: Optional[Union[Dict[str, List[str]], List[str]]] = field(
        default=None, repr=False
    )
    source: str = field(init=False)
    source_type: str = field(init=False)

    def __post_init__(self) -> None:
        """Post-init method."""
        self.source = "generic"
        self.source_type = "in_memory"
        if isinstance(self.obj, dict):
            if "transcript" not in self.obj:
                raise ValueError(
                    "Please provide a valid in-memory object. It must have a `transcript` key."
                )
            elif not isinstance(self.obj["transcript"], list):
                raise TypeError(
                    "Please provide a valid in-memory object. The `transcript` key must be a list."
                )
        elif isinstance(self.obj, list):
            self.obj = {"transcript": self.obj}
        else:
            raise TypeError(
                "Please provide a valid in-memory object. It must be a list or a dict."
            )

    def prepare_payload(self) -> str:
        """Prepare payload for API request."""
        self.payload = json.dumps(self.obj)
        return self.payload

    def prepare_headers(self) -> Dict[str, str]:
        """Prepare headers for API request."""
        self.headers = {
            "Accept": "application/json",
            "Content-Type": "application/json",
        }
        return self.headers


@dataclass
class GenericSource(BaseSource):
    """Generic source object.

    The GenericSource object is required to create a job that uses a generic file as input,
    such as `.txt` or `.json` file.

    Parameters
    ----------
    filepath : Union[str, Path]
        The path to the local file.
    url : str
        The URL to the remote file.

    Raises
    ------
    ValueError
        If the file format is not supported.
    ValueError
        If both `filepath` and `url` are provided.
    TypeError
        If the path is not a string or a Path object.
    FileNotFoundError
        If the file does not exist or is not accessible.

    Examples
    --------
    >>> from wordcab.core_objects import GenericSource

    >>> generic_source = GenericSource(filepath="path/to/generic/file.txt")  # doctest: +SKIP
    >>> generic_source  # doctest: +SKIP
    GenericSource(...)
    >>> generic_source.file_object  # doctest: +SKIP
    b'Hello, world!'
    >>> generic_source.source_type  # doctest: +SKIP
    'local'
    >>> generic_source._suffix  # doctest: +SKIP
    '.txt'
    >>> generic_source._stem  # doctest: +SKIP
    'file'

    Returns
    -------
    GenericSource
        The generic source object.
    """

    file_object: bytes = field(init=False, repr=False)

    def __post_init__(self) -> None:
        """Post-init method."""
        super().__post_init__()
        self.source = "generic"
        if self.source_type == "local":
            if self._suffix not in AVAILABLE_GENERIC_FORMATS:
                raise ValueError(
                    f"Please provide a valid file format. {self._suffix} is not valid."
                )
            else:
                self.file_object = self._load_file_from_path()

        if self.source_type == "remote":
            self.file_object = self._load_file_from_url()

    def prepare_payload(self) -> str:
        """Prepare payload for API request."""
        if self._suffix == ".json":
            self.payload = json.dumps({"transcript": json.loads(self.file_object)})
        elif self._suffix == ".txt":
            self.payload = json.dumps(
                {"transcript": self.file_object.decode("utf-8").splitlines()}
            )
        return self.payload

    def prepare_headers(self) -> Dict[str, str]:
        """Prepare headers for API request."""
        self.headers = {
            "Accept": "application/json",
            "Content-Type": "application/json",
        }
        return self.headers


@dataclass
class AudioSource(BaseSource):
    """
    The AudioSource object is required to create a job that uses an audio file as input.

    Parameters
    ----------
    filepath : Union[str, Path]
        The path to the local file.
    url : str
        The URL to the remote file.

    Raises
    ------
    ValueError
        If the file format is not supported.
    ValueError
        If both `filepath` and `url` are provided.
    TypeError
        If the path is not a string or a Path object.
    FileNotFoundError
        If the file does not exist or is not accessible.

    Examples
    --------
    >>> from wordcab.core_objects import AudioSource

    >>> audio_source = AudioSource(filepath="path/to/audio/file.mp3")  # doctest: +SKIP
    >>> audio_source  # doctest: +SKIP
    AudioSource(...)

    Returns
    -------
    AudioSource
        The audio source object.
    """

    file_object: bytes = field(init=False, repr=False)

    def __post_init__(self) -> None:
        """Post-init method."""
        super().__post_init__()
        self.source = "audio"
        if self.source_type == "local":
            if self._suffix not in AVAILABLE_AUDIO_FORMATS:
                raise ValueError(
                    f"Please provide a valid file format. {self._suffix} is not valid."
                )
            else:
                self.file_object = self._load_file_from_path()

        if self.source_type == "remote":
            self.file_object = self._load_file_from_url()

    def prepare_payload(self) -> Dict[str, bytes]:
        """Prepare payload for API request."""
        self.payload = {"audio_file": self.file_object}
        return self.payload

    @no_type_check
    def prepare_headers(self) -> dict:
        """Prepare headers for API request."""
        self.headers = {}
        return self.headers


@dataclass
class WordcabTranscriptSource(BaseSource):
    """Wordcab transcript source object."""

    transcript_id: Optional[str] = field(default=None)

    def __post_init__(self) -> None:
        """Post-init method."""
        super().__post_init__()
        if self.transcript_id is None:
            raise ValueError(
                "Please provide a `transcript_id` to initialize a WordcabTranscriptSource object."
            )
        self.source = "wordcab_transcript"
        raise NotImplementedError("Wordcab transcript source is not implemented yet.")


@dataclass
class SignedURLSource(BaseSource):
    """Signed URL source object."""

    signed_url: Optional[str] = field(init=False)

    def __post_init__(self) -> None:
        """Post-init method."""
        super().__post_init__()
        self.signed_url = self.url
        self.source = "signed_url"
        raise NotImplementedError("Signed URL source is not implemented yet.")


@dataclass
class AssemblyAISource(BaseSource):
    """AssemblyAI source object."""

    def __post_init__(self) -> None:
        """Post-init method."""
        super().__post_init__()
        self.source = "assembly_ai"
        raise NotImplementedError("AssemblyAI source is not implemented yet.")


@dataclass
class DeepgramSource(BaseSource):
    """Deepgram source object."""

    def __post_init__(self) -> None:
        """Post-init method."""
        super().__post_init__()
        self.source = "deepgram"
        raise NotImplementedError("Deepgram source is not implemented yet.")


@dataclass
class RevSource(BaseSource):
    """Rev.ai source object."""

    def __post_init__(self) -> None:
        """Post-init method."""
        super().__post_init__()
        self.source = "rev_ai"
        raise NotImplementedError("Rev.ai source is not implemented yet.")


@dataclass
class VTTSource(BaseSource):
    """VTT source object."""

    def __post_init__(self) -> None:
        """Post-init method."""
        super().__post_init__()
        self.source = "vtt"
        raise NotImplementedError("VTT source is not implemented yet.")



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['wordcab', 'wordcab.core_objects']

package_data = \
{'': ['*']}

install_requires = \
['click>=8.0.1', 'requests>=2.28.1', 'validators>=0.20.0']

entry_points = \
{'console_scripts': ['wordcab = wordcab.__main__:main']}

setup_kwargs = {
    'name': 'wordcab',
    'version': '0.3.2',
    'description': 'Wordcab Python SDK',
    'long_description': '# Wordcab Python\n\n[![PyPI](https://img.shields.io/pypi/v/wordcab.svg)][pypi_]\n[![Status](https://img.shields.io/pypi/status/wordcab.svg)][status]\n[![Python Version](https://img.shields.io/pypi/pyversions/wordcab)][python version]\n[![License](https://img.shields.io/pypi/l/wordcab)][license]\n\n[![Read the documentation at https://wordcab-python.readthedocs.io/](https://img.shields.io/readthedocs/wordcab-python/latest.svg?label=Read%20the%20Docs)][read the docs]\n[![Tests](https://github.com/Wordcab/wordcab-python/workflows/Tests/badge.svg)][tests]\n[![Codecov](https://codecov.io/gh/Wordcab/wordcab-python/branch/main/graph/badge.svg)][codecov]\n\n[![pre-commit](https://img.shields.io/badge/pre--commit-enabled-brightgreen?logo=pre-commit&logoColor=white)][pre-commit]\n[![Black](https://img.shields.io/badge/code%20style-black-000000.svg)][black]\n\n[pypi_]: https://pypi.org/project/wordcab-python/\n[status]: https://pypi.org/project/wordcab-python/\n[python version]: https://pypi.org/project/wordcab-python\n[read the docs]: https://wordcab-python.readthedocs.io/\n[tests]: https://github.com/Wordcab/wordcab-python/actions?workflow=Tests\n[codecov]: https://app.codecov.io/gh/Wordcab/wordcab-python\n[pre-commit]: https://github.com/pre-commit/pre-commit\n[black]: https://github.com/psf/black\n\n## Wordcab\n\n### What is Wordcab?\n\n**Summarize any business communications at scale with Wordcab\'s API.**\n\n**Wordcab** is a summarization service that provides a simple API to summarize any `audio`, `text`, or `JSON` file.\n\nIt also includes compatibility with famous transcripts platforms like [AssemblyAI](https://www.assemblyai.com/),\n[Deepgram](https://deepgram.com/), [Rev.ai](https://www.rev.ai/), [Otter.ai](https://otter.ai/), or\n[Sonix.ai](https://sonix.ai/).\n\n### Getting started\n\nYou can learn more about Wordcab services and pricing on [our website](https://wordcab.com/).\n\nIf you want to try out the API, you can [signup](https://wordcab.com/signup/) for a free account and start using the API\nright away.\n\n## Requirements\n\n- Os: Linux, Mac, Windows\n- Python 3.8+\n\n## Installation\n\nYou can install _Wordcab Python_ via [pip] from [PyPI]:\n\n```console\n$ pip install wordcab\n```\n\nStart using the API with any python script right away:\n\n```python\nfrom wordcab import get_stats\n\nstats = get_stats()\nprint(stats)\n```\n\n## Usage\n\n[<img src="https://cdn.loom.com/sessions/thumbnails/25150a30c593467fa1632145ff2dea6f-with-play.gif" width="50%">](https://www.loom.com/embed/25150a30c593467fa1632145ff2dea6f "Quick Python Package Demo")\n\nPlease see the [Documentation](https://wordcab-python.readthedocs.io/) for details.\n\n## Contributing\n\nContributions are very welcome. 🚀\nTo learn more, see the [Contributor Guide].\n\n## License\n\nDistributed under the terms of the [Apache 2.0 license][license],\n_Wordcab Python SDK_ is free and open source software.\n\n## Issues\n\nIf you encounter any problems,\nplease [file an issue] along with a detailed description.\n\n## Credits\n\nThis project was generated from [@cjolowicz]\'s [Hypermodern Python Cookiecutter] template.\n\n[@cjolowicz]: https://github.com/cjolowicz\n[pypi]: https://pypi.org/\n[hypermodern python cookiecutter]: https://github.com/cjolowicz/cookiecutter-hypermodern-python\n[file an issue]: https://github.com/Wordcab/wordcab-python/issues\n[pip]: https://pip.pypa.io/\n\n<!-- github-only -->\n\n[license]: https://github.com/Wordcab/wordcab-python/blob/main/LICENSE\n[contributor guide]: https://github.com/Wordcab/wordcab-python/blob/main/CONTRIBUTING.md\n[command-line reference]: https://wordcab-python.readthedocs.io/en/latest/usage.html\n',
    'author': 'Wordcab',
    'author_email': 'info@wordcab.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/Wordcab/wordcab-python',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
